#include "PCH.h"
#include "chara.h"
#include "map.h"
#include "crystal.h"
#include "lift.h"
#include "scene.h"

int Chara::rest_ = 3;
int Chara::yellow_ = 0;

//========================================================
Chara::Chara(int x, int y, Drct drct) :
	UQ::Task(UPDATE_CHARA, RENDER_CHARA),
	Object((float)x * 32 + (32 - 24) / 2, (float)y * 32 - (42 - 32), 24, 0, drct),
	status_(WALK),
	count_(0),
	pCarryCrystal_(0), pRideLift_(0),
	dashCount_(0),
	throwTransition_(NOT), throwCount_(0),
	green_(0),
	hp_(3),
	damageCount_(0), crystalCount_(0)
{
	setStatus(WALK);
	setScreenPos();
}

//========================================================
void Chara::setStatus(Status status)
{
	static const int HEIGHT[12] = {42, 40, 41, 32, 48, 42, 42, 42, 38, 42, 42, 42};
	height_ = HEIGHT[status];
	y_ += HEIGHT[status_] - HEIGHT[status];
	count_ = 0;
	status_ = status;
}

//========================================================
void Chara::holdLadder(void)
{
	if(pCarryCrystal_) return;
	if(!UQ::getIM()->isPressed(UQ::InputMgr::VC1_UP)) return;
	for(int i = 0; i < 2; i++){for(int j = 0; j < 3; j++){
		if(pMap_->getKind(x_ + (width_ - 1) * i, y_ + (height_ - 1) * j / 2) == Map::LADR){
			setStatus(LADDER); return;
		}
	}}
}

//========================================================
void Chara::beDamaged(float enemyX)
{
	if(damageCount_) return;
	UQ::getSM()->play(SND_DIE);
	hp_--;
	if(pCarryCrystal_){
		pCarryCrystal_->vx_ = (drct_ * 2 - 1) * 2.f; pCarryCrystal_->vy_ = 0.f;
		pCarryCrystal_->setStatus(Crystal::FALL);
		pCarryCrystal_ = 0;
		throwTransition_ = NOT; throwCount_ = 0;
	}
	if(hp_ == 0){
		setStatus(DIE);
	}else{
		vx_ = (x_ > enemyX) ? 1.f : -1.f;
		vy_ = 0.f;
		damageCount_ = 1;
		setStatus(DAMAGE);
	}
}

//========================================================
void Chara::LeftRight(void)
{
	static const float VX_MAX[6] = {2.5f, 4.0f, 1.0f, 0.5f, 2.5f, 2.0f};
	float vxMax = VX_MAX[status_];
	if(status_ == WALK && pMap_->isWater(x_ + width_ / 2, y_ + height_ - 1)) vxMax = 0.5f; // 水中は歩くの遅い

	float ax = 0.1f / ((pMap_->getKind(x_ + width_ / 2, y_ + height_) == Map::ICEB) ? 2 : 1);
	bool isKeyDown = false;
	if(UQ::getIM()->isDown(UQ::InputMgr::VC1_LEFT )){drct_ = LEFT ; isKeyDown = true;}
	if(UQ::getIM()->isDown(UQ::InputMgr::VC1_RIGHT)){drct_ = RIGHT; isKeyDown = true;}

	if(isKeyDown && status_ != BREAK){
		int drct = drct_ * 2 - 1;
		if(status_ == WALK || status_ == DASH){
			if(VX_MAX[WALK] - 1.f + drct * vx_ < 0.f){ // 反対方向に速すぎる = ブレーキ
				setStatus(BREAK); vx_ += drct * 1.f; UQ::getSM()->play(SND_BREAK);
			}
		}
		if(vxMax - drct * vx_ > 0.f){	// 加速の余地あり = 加速
			vx_ += drct * ax;
		}else{							// 進行方向に速すぎる = 減速
			if(status_ != JUMP) vx_ -= drct * ax;
		}

	}else{
		if(status_ != JUMP){
			vx_ -= sign(vx_) * ax * ((status_ == BREAK) ? 1.5f : 1);
			if(abs(vx_) < ax) vx_ = 0.f;
		}
		if(status_ == DASH) setStatus(WALK);
	}
}

//========================================================
void Chara::update(void)
{
	px_ = x_; py_ = y_;
	if(damageCount_ > 0){damageCount_++; if(damageCount_ >= 100) damageCount_ = 0;}

	switch(status_){
	case DAMAGE: // ダメージ-----------------------------------------------------------------------------------------
		count_++;
		if(count_ == 20) setStatus(WALK);
		break;

	case BREAK: // ブレーキ-----------------------------------------------------------------------------------------
		if(abs(vx_) < 0.1f) setStatus(WALK);
	// fall through.

	case WALK: // 歩く-----------------------------------------------------------------------
	case DASH: // ダッシュ-----------------------------------------------------------------------
		LeftRight();
		if(abs(vx_) >0.f) count_++;
		// ダッシュコマンド
		if(status_ == WALK && !pMap_->isWater(x_ + width_ / 2, y_ + height_ - 1)){
			static const int DASH_COUNT_MAX = 12;
			if(abs(dashCount_) > 0){
				dashCount_ += sign(dashCount_);
				if(abs(dashCount_) > DASH_COUNT_MAX) dashCount_ = 0;
			}
			if(UQ::getIM()->isPressed(UQ::InputMgr::VC1_LEFT) || UQ::getIM()->isPressed(UQ::InputMgr::VC1_RIGHT)){
				if(dashCount_ == 0){
					dashCount_ = drct_ * 2 - 1;
				}else if(sign(dashCount_) == drct_ * 2 - 1 && abs(dashCount_) <= DASH_COUNT_MAX){
					dashCount_ = 0;
					setStatus(DASH);
					vx_ += (drct_ * 2 - 1) * 1.f;
				}
			}
		}
		
		if(throwTransition_ != NOT) break;

		if(UQ::getIM()->isPressed(UQ::InputMgr::VC1_DOWN)) setStatus(SIT);
		holdLadder();
		// ジャンプ
		if(UQ::getIM()->isPressed(UQ::InputMgr::VC1_BTN1)){
			if(!pMap_->isWater(x_ + width_ / 2, y_)){
				vy_ = 0;
				setStatus(JUMP);
			}else{
				setStatus(SWIM); count_ = 1; vy_ = -3.f;
				UQ::getSM()->play(SND_SWIM);
			}
		}
		if(UQ::getIM()->isPressed(UQ::InputMgr::VC1_BTN2) && pCarryCrystal_) throwTransition_ = TAKE_BACK;
		// ドア
		if(UQ::getIM()->isPressed(UQ::InputMgr::VC1_UP) && pMap_->getKind(x_ + width_ / 2, y_ + height_ - 1) == Map::DOOR && !UQ::getTM()->get<Fade>()){
			vx_ = 0; vy_ = 0;
			if(pCarryCrystal_ && pMap_->getKind(x_ + width_ / 2, y_ + height_ - 32 - 1) == Map::DGOL) pCarryCrystal_->setStatus(Crystal::INVALID);
			UQ::getMM()->stop();
			setStatus(DOOR);
			UQ::getTM()->insert(new Fade(true));
		}
		break;

	case SIT: // しゃがむ-----------------------------------------------------------------------
		LeftRight();
		// 立つ
		if(!UQ::getIM()->isDown(UQ::InputMgr::VC1_DOWN) && (pMap_->getStatus(x_, y_ - 1).Thru && pMap_->getStatus(x_ + width_ - 1, y_ - 1).Thru)){
			setStatus(WALK);
		}
		break;

	case JUMP: // ジャンプ-----------------------------------------------------------------------------------------
		LeftRight();
		count_++;
		if(count_ == COUNT_PREJUMP_){
			UQ::getSM()->play(SND_JUMP);
			vy_ = -sqrt(2 * G_ * 135);
			if(pMap_->isWater(x_ + width_ / 2, y_ - 1)) vy_ = -2.f;
			pRideLift_ = 0;
		}else if(count_ > COUNT_PREJUMP_){
			vy_ += G_; if(vy_ > 10.f) vy_ = 10.f;
			if(!UQ::getIM()->isDown(UQ::InputMgr::VC1_BTN1) && vy_ < 0.f) vy_ = vy_ / 1.6f; // 飛距離調節

			if(throwTransition_ == NOT){
				holdLadder();
				if(UQ::getIM()->isPressed(UQ::InputMgr::VC1_BTN2) && pCarryCrystal_) throwTransition_ = TAKE_BACK;
			}
			// 水中へ
			if(pMap_->isWater(x_ + width_ / 2, y_ - 1)) setStatus(SWIM);
		}
		break;

	case SWIM: // 泳ぐ-----------------------------------------------------------------------------------------
		LeftRight();
		vy_ += G_ / 2.f; if(vy_ > 3.f) vy_ = 3.f;
		if(!pMap_->isWater(x_ + width_ / 2, y_ - 1) && vy_ < 0.f) vy_ /= 1.2f;
		
		if(throwTransition_ != NOT) break;

		if(UQ::getIM()->isPressed(UQ::InputMgr::VC1_BTN1)){
			if(pMap_->isWater(x_ + width_ / 2, y_)){
				vy_ = -3.f;
				if(count_ == 0) count_ = 1;
				UQ::getSM()->play(SND_SWIM);
			}else{
				if(UQ::getIM()->isDown(UQ::InputMgr::VC1_UP)){
					setStatus(JUMP); count_ = COUNT_PREJUMP_ - 1; break;
				}
			}
		}
		if(pCarryCrystal_){
			count_ = 0;
		}else{
			if(count_ > 0){
				count_++;
				if(count_ == 5 * 4) count_ = 0;
			}
		}
		holdLadder();
		if(UQ::getIM()->isPressed(UQ::InputMgr::VC1_BTN2) && pCarryCrystal_) throwTransition_ = TAKE_BACK;
		break;

	case LADDER:{ // はしご-----------------------------------------------------------------------
		vx_ = 0.f; vy_ = 0.f;
		if(UQ::getIM()->isDown(UQ::InputMgr::VC1_LEFT )) vx_ = -1.f;
		if(UQ::getIM()->isDown(UQ::InputMgr::VC1_RIGHT)) vx_ =  1.f;
		if(UQ::getIM()->isDown(UQ::InputMgr::VC1_UP   )) vy_ = -1.f;
		if(UQ::getIM()->isDown(UQ::InputMgr::VC1_DOWN )) vy_ =  1.f;
		if(abs(vx_) > 0 || abs(vy_) > 0) count_++;

		// はしごから落下
		bool isHolding = false;
		for(int i = 0; i < 2; i++){for(int j = 0; j < 3; j++){
			if(pMap_->getKind(x_ + (width_ - 1) * i, y_ + (height_ - 1) * j / 2) == Map::LADR){
				isHolding = true;
			}
		}}
		if(!isHolding || UQ::getIM()->isPressed(UQ::InputMgr::VC1_BTN1)){
			vy_ = 0.f;
			setStatus(JUMP); count_ = COUNT_PREJUMP_ + 1;
			if(hitsMapUp()) correctUp(true);
		}
		break;}

	case DIE: // 死-----------------------------------------------------------------------------------------
		if(count_ == 0){
			UQ::getSM()->play(SND_DIE);
			hp_ = 0;
			vx_ = 0.f; vy_ = -6.f;
			damageCount_ = 0;
			UQ::getMM()->stop();
			UQ::getSystem()->wait(500);
			UQ::getMM()->play(MSC_DIE, false);
		}
		count_++;
		vy_ += G_ / 2.f; y_ += vy_;
		if(vy_ > 10.f) vy_ = 10.f; // 打ち止め
		if(count_ == 180) UQ::getTM()->insert(new Fade(true));
		if(count_ > 180 && UQ::getTM()->get<Fade>()->isEnd()){
			UQ::getTM()->get<Map>()->erase();
			if(rest_ > 0){
				rest_--; 
				Map::goSelectStage();
			}else{ // ゲームオーバー
				UQ::getTM()->insert(new GameOver());
			}
		}
		return;

	case DOOR:{ // ドアin-----------------------------------------------------------------------------------------
		if(!UQ::getTM()->get<Fade>()->isEnd()) return;
		int dataLayer = pMap_->getLayerCount() - 1;
		int door = pMap_->getValue(dataLayer, x_ + width_ / 2, y_ + height_ - 1);
		if(isBetween<int>(door, Map::A1, Map::B2)){
			UQ::Vector2DTemplate<int> newPos;
			newPos = pMap_->enterDoor(door);
			x_ = newPos.x_ * 32.f + (32 - width_) / 2;
			y_ = newPos.y_ * 32.f - (height_ - 32);
			setScreenPos();
			setStatus(WALK);
		}else{
			switch(pMap_->getKind(x_ + width_ / 2, y_ + height_ - 32 - 1)){
			case Map::DNXT:
				gameInfo.world_++;
				pMap_->erase();
				UQ::getTM()->insert(new Map(Map::DPRV));
				break;
			case Map::DPRV:
				gameInfo.world_--;
				pMap_->erase();
				UQ::getTM()->insert(new Map(Map::DNXT));
				break;
			case Map::DGOL:
				pMap_->erase();
				UQ::getTM()->insert(new StageClear(pMap_->crystalCount_, crystalCount_));
				break;
			case Map::DST1: case Map::DST2: case Map::DST3: case Map::DST4: case Map::DST5:
				gameInfo.stage_ = pMap_->getKind(x_ + width_ / 2, y_ + height_ - 32 - 1) - Map::DST1 + 1;
				pMap_->erase();
				UQ::getTM()->insert(new StageStart());
				break;
			}
		}
		return;}

	case CLEAR:  // クリア時登場-----------------------------------------------------------------------------------------
		count_++;
		x_ += 1.5f;
		return;

	case TITLE:  // オープニング時登場-----------------------------------------------------------------------------------------
		//x_ += 1.5f;
		setScreenPos();
		return;
	}

	// 移動
	x_ += vx_; if(pRideLift_) x_ += pRideLift_->vx_;
	if(hitsMapLeft()) correctLeft(true);
	if(hitsMapRight()) correctRight(true);
	for(unsigned int i = 0; i < UQ::getTM()->count<Lift>(); i++){Lift* pLift = UQ::getTM()->enumerate<Lift>(i);
		if(!pMap_->getStatus(pLift->value_).Thru && pLift->hitsRect(x_, y_, 1, height_)){ // 左
			x_ = pLift->x_ + 32;
			vx_ = 0.f;
		}
		if(!pMap_->getStatus(pLift->value_).Thru && pLift->hitsRect(x_ + width_ - 1, y_, 1, height_)){ // 右
			x_ = pLift->x_ - width_;
			vx_ = 0.f;
		}
	}

	y_ += vy_; if(pRideLift_) y_ += pRideLift_->vy_;
	if(hitsMapUp()) correctUp(true);
	if(hitsMapDown()){correctDown(true); setStatus(WALK); pRideLift_ = 0;}
	for(unsigned int i = 0; i < UQ::getTM()->count<Lift>(); i++){Lift* pLift = UQ::getTM()->enumerate<Lift>(i);
		if(!pMap_->getStatus(pLift->value_).Thru && pLift->hitsRect(x_, y_, width_, 1)){ // 上
			y_ = pLift->y_ + 32;
			vy_ = 0;
		}
		if(pMap_->getStatus(pLift->value_).On && py_ + height_ - 5 < pLift->y_ && pLift->hitsRect(x_, y_ + height_ - 1, width_, 1)){ // 下
			y_ = pLift->y_ - height_;
			vy_ = 0.f; setStatus(WALK);
			pRideLift_ = pLift;
		}
	}

	// 落下
	if(status_ == WALK || status_ == DASH || status_ == BREAK || status_ == SIT){
		bool isFall = true;
		if(pRideLift_){
			for(unsigned int i = 0; i < UQ::getTM()->count<Lift>(); i++){Lift* pLift = UQ::getTM()->enumerate<Lift>(i);
				if(pLift->hitsRect(x_, y_ + height_, width_, 2)){
					isFall = false; break;
				}
			}
		}else{
			isFall = this->isFall();
		}
		if(isFall){vy_ = 0; y_++; setStatus(JUMP); count_ = COUNT_PREJUMP_ + 1; pRideLift_ = 0;}
	}

	// 投げ
	if(throwTransition_ == TAKE_BACK){
		throwCount_++;
		if(throwCount_ == 10){
			throwCount_ = 0;
			throwTransition_ = FOLLOW_THROUGH;
			pCarryCrystal_->setStatus(Crystal::THROWN);
			if(UQ::getIM()->isDown(UQ::InputMgr::VC1_UP)){
				pCarryCrystal_->vx_ = (drct_ * 2 - 1) * 6 * 0.7f;
				pCarryCrystal_->vy_ = -6 * 0.7f;
			}else if(UQ::getIM()->isDown(UQ::InputMgr::VC1_DOWN)){
				pCarryCrystal_->vx_ = (drct_ * 2 - 1) * 6 * 0.7f;
				pCarryCrystal_->vy_ = 6 * 0.7f;
			}else{
				pCarryCrystal_->y_ += 20.f;
				pCarryCrystal_->vx_ = (drct_ * 2 - 1) * 6.f;
				pCarryCrystal_->vy_ = 0.f;
			}
			pCarryCrystal_ = 0;
		}
	}else if(throwTransition_ == FOLLOW_THROUGH){
		throwCount_++;
		if(throwCount_ == 15){throwTransition_ = NOT; throwCount_ = 0;}
	}

	// 持ってるアイテムを移動
	if(pCarryCrystal_){
		pCarryCrystal_->x_ = x_ + width_ / 2 - pCarryCrystal_->width_ / 2;
		if(throwTransition_ == TAKE_BACK) pCarryCrystal_->x_ -= (drct_ * 2 - 1) * 10;
		pCarryCrystal_->y_ = y_ - pCarryCrystal_->height_;
		if(status_ == JUMP && count_ <= COUNT_PREJUMP_) pCarryCrystal_->y_ += 12;
	}

	// 泡発生
	if(UQ::getSystem()->getFrameCount() % 100 == 0 && pMap_->isWater(x_ + width_ / 2, y_)){
		UQ::getTM()->insert(new Bubble(this), pMap_);
	}

	// ポーズ
	if(UQ::getIM()->isPressed(UQ::InputMgr::VC1_BTN3)){
		UQ::getTM()->insert(new Pause());
	}

	// 穴に落ちて死
	if(y_ >= pMap_->getHeight() * 32 + 32){
		pCarryCrystal_ = 0;
		setStatus(DIE);
	}
	// 時間切れ
	if(!gameInfo.isStageSelect() && UQ::getTM()->get<Timer>()->getRestTime() == 0){
		setStatus(DIE);
	}

	setScreenPos();
}

//========================================================
void Chara::setScreenPos(void)
{
	pMap_->left_ = x_ + width_ / 2 - SCREEN_WIDTH / 2;
	pMap_->top_ = y_ + height_ - SCREEN_HEIGHT / 2;

	int dataLayer = pMap_->getLayerCount() - 1;
	for(int i = 0; i <= SCREEN_WIDTH / 32; i++){
	for(int j = (int)pMap_->top_ / 32; j <= ((int)pMap_->top_  + SCREEN_HEIGHT) / 32; j++){
		int x = (int)pMap_->left_ / 32 + SCREEN_WIDTH / 2 / 32 + (i + 1) / 2 * (((i / 2) * 2 == i) ? -1 : 1); // 真ん中から左右交互に見ていく
		int value = pMap_->getValue(dataLayer, x, j);
		if(value == Map::NOSCROLL_LEFT){pMap_->left_ = (float)(x + 1) * 32; goto END_X;}
		if(value == Map::NOSCROLL_RIGHT){pMap_->left_ = (float)x * 32 - SCREEN_WIDTH; goto END_X;}
	}}
END_X:
	for(int j = 0; j <= SCREEN_HEIGHT / 32; j++){
	for(int i = (int)pMap_->left_ / 32; i <= ((int)pMap_->left_ + SCREEN_WIDTH) / 32; i++){
		int y = (int)pMap_->top_ / 32 + SCREEN_HEIGHT / 2 / 32 + (j + 1) / 2 * (((j / 2) * 2 == j) ? -1 : 1); // 同上
		int value = pMap_->getValue(dataLayer, i, y);
		if(value == Map::NOSCROLL_UP){pMap_->top_ = (float)(y + 1) * 32; goto END_Y;}
		if(value == Map::NOSCROLL_DOWN){pMap_->top_ = (float)y * 32 - SCREEN_HEIGHT; goto END_Y;}
	}}
END_Y:

	if((int)pMap_->left_ < 0) pMap_->left_ = 0.f;
	if((int)pMap_->left_ + SCREEN_WIDTH > pMap_->getWidth() * 32) pMap_->left_ = (float)pMap_->getWidth() * 32 - SCREEN_WIDTH;
	if((int)pMap_->top_ < 0) pMap_->top_ = 0.f;
	if((int)pMap_->top_ + SCREEN_HEIGHT > pMap_->getHeight() * 32) pMap_->top_ = (float)pMap_->getHeight() * 32 - SCREEN_HEIGHT;
}

//========================================================
void Chara::render(void)
{
	UQ::getGM()->setTexture(TEX_CHARA);

	UQ::Polygon rect;
	rect.setColor((damageCount_ / 2 % 2 == 1) ? 0x60FFFF80 : 0xFFFFFFFF);

	rect.setRectangle(
		x_ - (32 - width_) / 2 - ((status_ == CLEAR) ? 0 : pMap_->left_),
		y_ - (48 - height_) - ((status_ == CLEAR) ? 0 : pMap_->top_),
		32, 48);

	int index = 0;
	switch(status_){
		case WALK: index = (count_ / 6) % 2; break;
		case DASH: index = (count_ / 4) % 2; break;
		case BREAK: index = 0; break;
		case SIT: index = 0; break;
		case JUMP: index = (count_ > COUNT_PREJUMP_) ? 1 : 0; break;
		case SWIM: index = (count_ / 5) % 4; break;
		case LADDER: index = (count_ / 6) % 2; break;
		case DAMAGE: index = 0; break;
		case DIE: index = 0; break;
		case DOOR: index = 0; break;
		case CLEAR: index = (count_ / 6) % 2; break;
		case TITLE: index = (count_ / 6) % 2; break;
	}
	rect.setRectangleTexture(
		drct_ * 256 + 
			((throwTransition_ == NOT) ? 
				index * 32 + ((pCarryCrystal_) ? 128 : 0)
				: 192 + (throwTransition_ - 1) * 32),
		(status_ == CLEAR || status_ == TITLE) ? 0 : status_ * 48,
		32, 48);

	UQ::getGM()->drawPolygon(rect);
}

//========================================================
//========================================================
Bubble::Bubble(Chara* pChara) :
	UQ::Task(UPDATE_BUBBLE, RENDER_BUBBLE),
	Object(pChara->x_ + pChara->width_ / 2 + (pChara->drct_ * 2 - 1) * 10 - 8, pChara->y_ + 10, 16, 16, LEFT),
	count_(0)
{
}

void Bubble::update(void)
{
	count_++;
	x_ += sin(count_ / 5.f);
	y_ -= 2.f;
	if(!pMap_->isWater(x_ + width_ / 2, y_)) erase();
}

void Bubble::render(void)
{
	UQ::getGM()->setTexture(TEX_CHARA);
	UQ::Polygon rect;
	rect.setColor(0xA0FFFFFF);
	rect.setRectangle(x_ - pMap_->left_, y_ - pMap_->top_, 16, 16);
	rect.setRectangleTexture(496, 496, 16, 16);
	UQ::getGM()->drawPolygon(rect);
}