#include "PCH.h"
#include "crystal.h"
#include "map.h"
#include "chara.h"
#include "enemy.h"

//========================================================
Crystal::Crystal(Enemy *pEnemy) :
	UQ::Task(UPDATE_CRYSTAL, RENDER_CRYSTAL),
	Object(pEnemy->x_, pEnemy->y_, 22, 22, LEFT),
	color_(pEnemy->color_),
	count_(0)
{
	vx_ = pEnemy->vx_ / 2; vy_ = pEnemy->vy_ / 2;
	if(color_ == GRAY){setStatus(INVALID);}else{setStatus(FALL);}
}

Crystal::Crystal(Color color, int x, int y) :
	UQ::Task(UPDATE_CRYSTAL, RENDER_CRYSTAL),
	Object(x * 32.f + (32 - 22) / 2, y * 32.f + (32 - 22) / 2, 22, 22, LEFT),
	color_(color),
	count_(0)
{
	setStatus(SET);
}

void Crystal::setStatus(Status status)
{
	status_ = status;
	if(status == THROWN || status == FALL || status == INVALID){
		count_ = 0x7FFFFFFF / 2;
	}else{
		count_ = 0;
	}
	if(status == INVALID || status == GOTTEN){
		UQ::getTM()->get<Chara>()->crystalCount_++;
	}
}

void Crystal::update(void)
{
	px_ = x_; py_ = y_;

	switch(status_){
	//case CARRIED:
	//case LIE:
	//break;

	case FALL:
	case THROWN:
		count_ += (vx_ > 0) ? 1 : -1;
		if(status_ != THROWN) vy_ += G_;

		// X
		x_ += vx_ / ((pMap_->isWater(x_, y_)) ? 2.f : 1.f);
		if(hitsMapLeft()){
			correctLeft(false); vx_ = - vx_ / 2.f;
			if(status_ == THROWN) setStatus(INVALID);
		}
		if(hitsMapRight()){
			correctRight(false); vx_ = - vx_ / 2.f;
			if(status_ == THROWN) setStatus(INVALID);
		}

		// Y
		y_ += vy_ / ((pMap_->isWater(x_, y_)) ? 2.f : 1.f);
		if(hitsMapUp()){
			correctUp(false); vy_ = - vy_ / 2.f;
			if(status_ == THROWN) setStatus(INVALID);
		}
		if(hitsMapDown()){
			correctDown(false); vy_ = - vy_ / 2.f;
			if(status_ == FALL){
				if(abs(vy_) < 1.f) setStatus(LIE);
			}
			if(status_ == THROWN) setStatus(INVALID);
		}
		break;

	case INVALID:
		count_ += (vx_ > 0) ? 1 : -1;
		vy_ += G_;
		x_ += vx_; y_ += vy_;
		break;

	case GOTTEN:
		count_++;
		y_ -= 3.f;
		if(count_ >= 30) erase();
		break;

	case SET:
		if(UQ::getSystem()->getFrameCount() % 4 == 0) count_++;
		break;
	}
	
	// 画面外で消滅
	if(status_ != SET){
		if(!hitsRect(pMap_->left_ - SCREEN_EXPANSION_, pMap_->top_ - SCREEN_EXPANSION_, SCREEN_WIDTH + SCREEN_EXPANSION_ * 2, SCREEN_HEIGHT + SCREEN_EXPANSION_ * 2)){
			if(status_ == THROWN) setStatus(INVALID);
			erase();
		}
	}
}

//========================================================
void Crystal::render()
{
	UQ::getGM()->setTexture(TEX_ENEMY);
	UQ::Polygon rect;
	rect.setColor((status_ == INVALID || status_ == GOTTEN)? 0x80FFFFFF : 0xFFFFFFFF);
	rect.setRectangle(x_ - (32 - width_) / 2 - pMap_->left_, y_ - (32 - height_) / 2 - pMap_->top_, 32, 32);
	rect.setRectangleTexture(color_ * 32, (count_ % 4) * 32, 32, 32);
	UQ::getGM()->drawPolygon(rect);
}