#include "PCH.h"
#include "enemy.h"
#include "map.h"
#include "chara.h"
#include "crystal.h"
#include "scene.h"
#include <cstdlib>

//========================================================
EnemyEmerge::EnemyEmerge(int x, int y, Kind kind, Color color, Object::Drct drct) :
	UQ::Task(UPDATE_ENEMY_EMERGE),
	x_(x), y_(y),
	kind_(kind),
	color_(color),
	drct_(drct),
	status_(NONE)
{
}

void EnemyEmerge::update(void)
{
	Map* pMap = UQ::getTM()->get<Map>();
	if(hits(x_ * 32.f, y_ * 32.f, 32, 32, pMap->left_ - 32, pMap->top_ - 32, SCREEN_WIDTH + 64, SCREEN_HEIGHT + 64) && status_ == NONE){
		Enemy* pEnemy = 0;
		switch(kind_){
			case WING1:    pEnemy = new Wing1(this);    break;
			case TORPEDO1: pEnemy = new Torpedo1(this); break;
			case TIRE:     pEnemy = new Tire(this);     break;
			case SAW:      pEnemy = new Saw(this);      break;
			case BOSS1:    pEnemy = new Boss1(this);    break;
		}
		UQ::getTM()->insert(pEnemy, pMap);
		status_ = EXIST;
		if(kind_ >= BOSS1){
			UQ::getMM()->stop();
			UQ::getTM()->get<Timer>()->setBgm(MSC_BOSS);
		}
	}
	// 一度出現したら遠くに行かないと再出現しない
	if(status_ == EXTINCT){
		if(!hits(x_ * 32.f, y_ * 32.f, 32, 32, pMap->left_ - 32, pMap->top_ - 32, SCREEN_WIDTH + 64, SCREEN_HEIGHT + 64)) status_ = NONE;
	}
}

//========================================================
Enemy::Enemy(EnemyEmerge* pEnemyEmerge) :
	UQ::Task(UPDATE_ENEMY, RENDER_ENEMY),
	Object(pEnemyEmerge->x_ * 32.f, pEnemyEmerge->y_ * 32.f, 24, 24, pEnemyEmerge->drct_),
	status_(EXIST),
	color_(pEnemyEmerge->color_),
	count_(0),
	pEnemyEmerge_(pEnemyEmerge)
{
}
void Enemy::update(void)
{
	px_ = x_; py_ = y_;

	if(!hitsRect(pMap_->left_ - SCREEN_EXPANSION_, pMap_->top_ - SCREEN_EXPANSION_, SCREEN_WIDTH + SCREEN_EXPANSION_ * 2, SCREEN_HEIGHT + SCREEN_EXPANSION_ * 2)){
		if(status_ != DIE){
			pEnemyEmerge_->status_ = EnemyEmerge::EXTINCT;
		}else{
			pEnemyEmerge_->erase();
		}
		erase();
	}
}
void Enemy::drawCrystal(int index, int shiftX, int shiftY)
{
	if(status_ != EXIST) return;
	UQ::getGM()->setTexture(TEX_ENEMY);
	UQ::Polygon rect;
	float x = x_ - (32 - width_) / 2 - pMap_->left_;
	float y = y_ - (32 - height_) / 2 - pMap_->top_;
	rect.setRectangle(x + shiftX, y + shiftY, 32, 32);
	rect.setRectangleTexture(color_ * 32, index * 32, 32, 32);
	UQ::getGM()->drawPolygon(rect);
}
void Enemy::drawKind(int kind, int index, int shiftX, int shiftY)
{
	UQ::getGM()->setTexture(TEX_ENEMY);
	UQ::Polygon rect;
	rect.setColor((status_ == DIE) ? 0x60FFFFFF : 0xFFFFFFFF);
	int drct = drct_ * 2 - 1;
	float x = x_ - (32 - width_) / 2 - pMap_->left_;
	float y = y_ - (32 - height_) / 2 - pMap_->top_;
	rect.setRectangle(x - shiftX * drct, y + shiftY, 32, 32);
	rect.setRectangleTexture(kind * 32, index * 32 + 128, 32, 32);
	UQ::getGM()->drawPolygon(rect);
}
void Enemy::drawEye(int shiftX, int shiftY, int drct)
{
	UQ::getGM()->setTexture(TEX_ENEMY);
	UQ::Polygon rect;
	rect.setColor((status_ == DIE) ? 0x60FFFFFF : 0xFFFFFFFF);
	if(drct == 0x4000) drct = drct_ * 2 - 1;
	float x = x_ - (32 - width_) / 2 - pMap_->left_;
	float y = y_ - (32 - height_) / 2 - pMap_->top_;
	rect.setRectangle(x + 16 + (3 + 4) * drct - 4 + shiftX, y + shiftY, 8, 16);
	rect.setRectangleTexture((drct + 1) / 2 * 8, 384 + ((status_ == DIE) ? 16 : 0), 8, 16);
	UQ::getGM()->drawPolygon(rect);
}
void Enemy::beDamaged(void)
{
	UQ::getSM()->play(SND_KILL);
	status_ = DIE;
	UQ::getTM()->insert(new Crystal(this), pMap_);
}

//========================================================
void Wing1::update(void)
{
	Enemy::update();
	switch(status_){
	case EXIST:
		count_++;
		y_ += sin(count_ / 20.f);
		drct_ = (UQ::getTM()->get<Chara>()->x_ - x_ > 0) ? RIGHT : LEFT;
		break;
	case DIE:
		count_++;
		vy_ -= 0.2f;
		x_ += 3 * (drct_ * 2 - 1); y_ += vy_;
		break;
	}
}

//========================================================
void Torpedo1::update(void)
{
	Enemy::update();
	switch(status_){
	case EXIST:
		count_++;
		x_ += (drct_ * 2 - 1) * 3;
		break;
	case DIE:
		count_++;
		vy_ -= 0.15f;
		x_ += 3 * (drct_ * 2 - 1); y_ += vy_;
		break;
	}
}

//========================================================
void Tire::update(void)
{
	Enemy::update();
	switch(status_){
	case EXIST:
		count_++;
		x_ += (drct_ * 2 - 1) * 1;
		if(hitsMapLeft()){correctLeft(false); drct_ = RIGHT;}
		if(hitsMapRight()){correctRight(false); drct_ = LEFT;}
		if(isWalking_){
			if(isFall()){vy_ = 0; y_++; isWalking_ = false;}
		}else{
			vy_ += G_; y_ += vy_;
			if(hitsMapDown()){correctDown(true); isWalking_ = true;}
		}
		break;
	case DIE:
		count_++;
		vy_ += 0.2f;
		x_ += 1 * (drct_ * 2 - 1); y_ += vy_;
		break;
	}
}

//========================================================
void Saw::update(void)
{
	Enemy::update();
	switch(status_){
	case EXIST:
		count_++;
		if(!isOn_){
			y_ += 2;
			if(hitsMapDown()){correctDown(true); isOn_ = true;}
			break;
		}
		switch(drct_){
		case DOWN:
			y_ += 2;
			if(drctOn_ == LEFT){
				if(pMap_->getStatus(x_ - 1, y_).Thru && pMap_->getStatus(x_ - 1, y_ + height_ - 1).Thru){ // 左
					y_ = floor(y_ / 32) * 32; drctOn_ = UP; drct_ = LEFT;
				}
				if(!pMap_->getStatus(x_, y_ + height_ - 1).Thru || !pMap_->getStatus(x_ + width_ - 1, y_ + height_ - 1).Thru){ // 下
					y_ = (floor((y_ + height_ - 1) / 32)) * 32 - height_; drctOn_ = DOWN; drct_ = RIGHT;
				}
			}else{
				if(pMap_->getStatus(x_ + width_, y_).Thru && pMap_->getStatus(x_ + width_, y_ + height_ - 1).Thru){ // 右
					y_ = floor(y_ / 32) * 32; drctOn_ = UP; drct_ = RIGHT;
				}
				if(!pMap_->getStatus(x_, y_ + height_ - 1).Thru || !pMap_->getStatus(x_ + width_ - 1, y_ + height_ - 1).Thru){ // 下
					y_ = (floor((y_ + height_ - 1) / 32)) * 32 - height_; drctOn_ = DOWN; drct_ = LEFT;
				}									
			}
			break;
		case UP:
			y_ -= 2;
			if(drctOn_ == LEFT){
				if(pMap_->getStatus(x_ - 1, y_).Thru && pMap_->getStatus(x_ - 1, y_ + height_ - 1).Thru){ // 左
					y_ = (floor((y_ + height_ - 1) / 32) + 1) * 32 - height_; drctOn_ = DOWN; drct_ = LEFT;
				}
				if(!pMap_->getStatus(x_, y_).Thru || !pMap_->getStatus(x_ + width_ - 1, y_).Thru){ // 上
					y_ = (floor(y_ / 32) + 1) * 32; drctOn_ = UP; drct_ = RIGHT;
				}
			}else{
				if(pMap_->getStatus(x_ + width_, y_).Thru && pMap_->getStatus(x_ + width_, y_ + height_ - 1).Thru){ // 右
					y_ = (floor((y_ + height_ - 1) / 32) + 1) * 32 - height_; drctOn_ = DOWN; drct_ = RIGHT;
				}
				if(!pMap_->getStatus(x_, y_).Thru || !pMap_->getStatus(x_ + width_ - 1, y_).Thru){ // 上
					y_ = (floor(y_ / 32) + 1) * 32; drctOn_ = UP; drct_ = LEFT;
				}									
			}
			break;
		case LEFT:
			x_ -= 2;
			if(drctOn_ == DOWN){
				if(pMap_->getStatus(x_, y_ + height_).Thru && pMap_->getStatus(x_ + width_ - 1, y_ + height_).Thru){ // 下
					x_ = (floor((x_ + width_ - 1) / 32) + 1) * 32 - width_; drctOn_ = RIGHT; drct_ = DOWN;
				}
				if(!pMap_->getStatus(x_, y_).Thru || !pMap_->getStatus(x_, y_ + height_ - 1).Thru){ // 左
					x_ = (floor(x_ / 32) + 1) * 32; drctOn_ = LEFT; drct_ = UP;
				}
			}else{
				if(pMap_->getStatus(x_, y_ - 1).Thru && pMap_->getStatus(x_ + width_ - 1, y_ - 1).Thru){ // 上
					x_ = (floor((x_ + width_ - 1) / 32) + 1) * 32 - width_; drctOn_ = RIGHT; drct_ = UP;
				}
				if(!pMap_->getStatus(x_, y_).Thru || !pMap_->getStatus(x_, y_ + height_ - 1).Thru){ // 左
					x_ = (floor(x_ / 32) + 1) * 32; drctOn_ = LEFT; drct_ = DOWN;
				}									
			}
			break;
		case RIGHT:
			x_ += 2;
			if(drctOn_ == DOWN){
				if(pMap_->getStatus(x_, y_ + height_).Thru && pMap_->getStatus(x_ + width_ - 1, y_ + height_).Thru){ // 下
					x_ = floor(x_ / 32) * 32; drctOn_ = LEFT; drct_ = DOWN;
				}
				if(!pMap_->getStatus(x_ + width_ - 1, y_).Thru || !pMap_->getStatus(x_ + width_ - 1, y_ + height_ - 1).Thru){ // 右
					x_ = floor((x_ + width_ - 1) / 32) * 32 - width_; drctOn_ = RIGHT; drct_ = UP;
				}
			}else{
				if(pMap_->getStatus(x_, y_ - 1).Thru && pMap_->getStatus(x_ + width_ - 1, y_ - 1).Thru){ // 上
					x_ = floor(x_ / 32) * 32; drctOn_ = LEFT; drct_ = UP;
				}
				if(!pMap_->getStatus(x_ + width_ - 1, y_).Thru || !pMap_->getStatus(x_ + width_ - 1, y_ + height_ - 1).Thru){ // 右
					x_ = floor((x_ + width_ - 1) / 32) * 32 - width_; drctOn_ = RIGHT; drct_ = DOWN;
				}									
			}
			break;
		}
		break;
	case DIE:
		count_++;
		vy_ += G_ / 2.f; y_ += vy_;
		break;
	}
}

//========================================================
void Boss::update(void)
{
	Enemy::update();
	switch(status_){
	case EXIST:
		if(damageCount_ > 0){damageCount_++; if(damageCount_ >= 100) damageCount_ = 0;}
		break;
	case DIE:
		if(count_ == 0){
			UQ::getMM()->stop();
			UQ::getSystem()->wait(500);
			UQ::getTM()->get<Timer>()->pauseUpdate();
		}
		if(count_ == 180) UQ::getTM()->insert(new Fade(true));
		if(count_ > 180 && UQ::getTM()->get<Fade>()->isEnd()){
			UQ::getTM()->get<Map>()->erase();
			UQ::getTM()->insert(new StageClear(pMap_->crystalCount_, UQ::getTM()->get<Chara>()->crystalCount_));
		}
		count_++;
		break;
	}
}
void Boss::drawCrystal(int index, int shiftX, int shiftY)
{
	UQ::getGM()->setTexture(TEX_ENEMY);
	UQ::Polygon rect;
	rect.setColor((status_ == DIE || damageCount_ / 2 % 2 == 1) ? 0x60FFFFFF : 0xFFFFFFFF);
	float x = x_ - (64 - width_) / 2 - pMap_->left_;
	float y = y_ - (64 - height_) / 2 - pMap_->top_;
	rect.setRectangle(x, y - 4, 64, 64);
	rect.setRectangleTexture(color_ * 64 + 256, index * 64, 64, 64);
	UQ::getGM()->drawPolygon(rect);
}
void Boss::drawKind(int kind, int index, int shiftX, int shiftY)
{
	UQ::getGM()->setTexture(TEX_ENEMY);
	UQ::Polygon rect;
	rect.setColor((status_ == DIE || damageCount_ / 2 % 2 == 1) ? 0x60FFFFFF : 0xFFFFFFFF);
	int drct = drct_ * 2 - 1;
	float x = x_ - (64 - width_) / 2 - pMap_->left_;
	float y = y_ - (64 - height_) / 2 - pMap_->top_;
	rect.setRectangle(x - shiftX * drct, y + shiftY, 64, 32);
	rect.setRectangleTexture(320, index * 32, 64, 32);
	UQ::getGM()->drawPolygon(rect);
}
void Boss::drawEye(int shiftX, int shiftY)
{
	UQ::getGM()->setTexture(TEX_ENEMY);
	UQ::Polygon rect;
	rect.setColor((status_ == DIE) ? 0x60FFFFFF : 0xFFFFFFFF);
	int drct = drct_ * 2 - 1;
	float x = x_ - (64 - width_) / 2 - pMap_->left_;
	float y = y_ - (64 - height_) / 2 - pMap_->top_;
	rect.setRectangle(x + 32 + (8 + 8) * drct - 8 + shiftX, y + shiftY, 16, 32);
	rect.setRectangleTexture(drct_ * 16, 416 + (status_ == DIE || (damageCount_ > 0 && damageCount_ < 50)) * 32, 16, 32);
	UQ::getGM()->drawPolygon(rect);
}
void Boss::drawExplosion(void)
{
	if(status_ != DIE) return;
	UQ::getGM()->unsetTexture();
	float x = x_ - (64 - width_) / 2 - pMap_->left_ + width_ / 2;
	float y = y_ - (64 - height_) / 2 - pMap_->top_ + height_ / 2;
	static const int COUNT_MAX = 60;
	if(count_ % COUNT_MAX == 1) UQ::getSM()->play(SND_EXPLODE);
	UQ::Polygon explosion;
	explosion.setColor(UQ::Color(0xFF - (BYTE)((float)(count_ % COUNT_MAX) / COUNT_MAX * 0xFF), 0xFF, 0xFF, 0xFF));
	explosion.setRegularPolygon(8, UQ::Vector2D(x, y), (count_ % COUNT_MAX) * 8.f);
	explosion.rotate(UQ::Vector2D(x, y), count_ / 20.f);
	UQ::getGM()->drawPolygon(explosion);
}
void Boss::beDamaged(void)
{
	UQ::getSM()->play(SND_KILL);
	hp_--;
	if(hp_ == 0){status_ = DIE; count_ = 0;}
	damageCount_ = 1;
}

//========================================================
void Boss1::update(void)
{
	Boss::update();
	switch(status_){
	case EXIST:
		count_++;
		switch(action_){
		case JUMP:
			vy_ += G_; y_ += vy_;
			if(hitsMapDown()){correctDown(true); action_ = WALK;}
			x_ += (drct_ * 2 - 1) * 2;
			if(hitsMapLeft()){correctLeft(false); drct_ = RIGHT;}
			if(hitsMapRight()){correctRight(false); drct_ = LEFT;}
			break;
		case WALK:
			x_ += (drct_ * 2 - 1) * 2;
			if(hitsMapLeft()){correctLeft(false); drct_ = RIGHT;}
			if(hitsMapRight()){correctRight(false); drct_ = LEFT;}
			if(isFall()){vy_ = 0; y_++; action_ = JUMP;}
			if(rand() % 60 == 0){vy_ = -7; action_ = JUMP;}
			if(rand() % 150 == 50){count_ = 0; action_ = RUSH; vx_ = (drct_ * 2 - 1) * 7.f;}
			break;
		case RUSH:
			if(count_ >= 30){
				vx_ -= (drct_ * 2 - 1) / 20.f; x_ += vx_;
				if(hitsMapLeft()){correctLeft(false); drct_ = RIGHT; vx_ = -vx_;}
				if(hitsMapRight()){correctRight(false); drct_ = LEFT; vx_ = -vx_;}
			}
			if(abs(vx_) < 1.f){action_ = WALK;} 
			break;
		}
		break;
	//case DIE:
	//	break;
	}
}
