#pragma once

#include "object.h"

//========================================================
class EnemyEmerge : public UQ::Task{
public:
	enum Status{NONE, EXIST, EXTINCT};
	enum Kind{WING1 = 0, TORPEDO1, TIRE, SAW, BOSS1 = 7};
public:
	EnemyEmerge(int x, int y, Kind kind, Color color, Object::Drct drct);
	void update(void);

public:
	int x_, y_;
	Kind kind_;
	Color color_;
	Object::Drct drct_;
	Status status_;
};

//========================================================
class Enemy : public UQ::Task, public Object{
public:
	Enemy(EnemyEmerge* pEnemyEmerge);
	virtual ~Enemy(){}
	virtual void update(void);
	virtual void drawCrystal(int index, int shiftX, int shiftY); 
	virtual void drawKind(int kind, int index, int shiftX, int shiftY);
	virtual void drawEye(int shiftX, int shiftY, int drct = 0x4000);
	virtual void beDamaged(void);
	virtual bool canHit(void){return (status_ != DIE);}
	virtual bool canBeTroddenOn(void){return true;} // 踏めない場合オーバーライド
public:
	enum{DIE, EXIST} status_;
	Color color_;
	int count_;
	EnemyEmerge* pEnemyEmerge_;
};

class Wing1 : public Enemy{
public:
	Wing1(EnemyEmerge* pEnemyEmerge) : Enemy(pEnemyEmerge){}
	void update(void);
	void render(void)
	{
		drawCrystal(2, 0, 0);
		drawKind(0, abs(count_ / 4 % 6 - 3) + drct_ * 4, 18, -3);
		drawEye(0, 2);
	}
};

class Torpedo1 : public Enemy{
public:
	Torpedo1(EnemyEmerge* pEnemyEmerge) : Enemy(pEnemyEmerge){}
	void update(void);
	void render(void)
	{
		drawCrystal(0, 0, 0);
		drawKind(1, abs(count_ / 3 % 6 - 3) + drct_ * 4, 26, 0);
		drawEye(0, 4);
	}
};

class Tire : public Enemy{
public:
	Tire(EnemyEmerge* pEnemyEmerge) : Enemy(pEnemyEmerge), isWalking_(false){}
	void update(void);
	void render(void)
	{
		drawCrystal(3 - drct_ * 2, 0, -4);
		drawKind(2, count_ / 5 % 4 + drct_ * 4, 18, -4);
		drawEye(0, -1);
	}
private:
	bool isWalking_;
};

class Saw : public Enemy{
public:
	Saw(EnemyEmerge* pEnemyEmerge) : Enemy(pEnemyEmerge), isOn_(false), drctOn_(DOWN){}
	void update(void);
	void render(void)
	{
		drawCrystal(count_ / 4 % 4, 0, 0);
		drawKind(3, abs(count_ / 2 % 14 - 7), 0, 0);
		drawEye(-14, 6, 1);
		drawEye(14, 6, -1);
	}
	bool canBeTroddenOn(void){return false;} // 踏めない
private:
	bool isOn_;
	Drct drctOn_;
};

//========================================================
class Boss : public Enemy{
public:
	Boss(EnemyEmerge* pEnemyEmerge) : Enemy(pEnemyEmerge), hp_(5), damageCount_(0){width_ = 56; height_ = 56;}
	virtual ~Boss(){}
	virtual void update(void);
	void drawCrystal(int index, int shiftX, int shiftY);
	void drawKind(int kind, int index, int shiftX, int shiftY);
	void drawEye(int shiftX, int shiftY);
	void drawExplosion(void);
	void beDamaged(void);
	bool canHit(void){return (status_ != DIE && damageCount_ == 0);}
private:
	int hp_;
	int damageCount_;
};

class Boss1 : public Boss{
public:
	Boss1(EnemyEmerge* pEnemyEmerge) : Boss(pEnemyEmerge), action_(JUMP){}
	void update(void);
	void render(void)
	{
		drawCrystal(count_ / 4 % 8, 0, -4);
		drawKind(0, count_ * ((action_ == 2) ? 2 : 1) / 5 % 4 + drct_ * 4, 7, -4 + 32);
		drawEye(0, 0);
		drawExplosion();
	}
private:
	enum{JUMP, WALK, RUSH} action_;
};
