#include "PCH.h"
#include "scene.h"

GameInfo gameInfo;
void initializeGame(void);
//==============================================================================================
// エントリー
//==============================================================================================
int main(int, char**)
{
	try{
		UQ::getSystem()->initialize(
			SCREEN_WIDTH, SCREEN_HEIGHT,
			"ばねばねくん",
			"image\\systemfont.png", 8, 16
		);
		initializeGame();
		UQ::getSystem()->run();

		UQ::getSystem()->deleteInstance();
		return 0;

	}catch(std::exception& ex){
		UQ::showError(ex);
		UQ::getSystem()->deleteInstance();
		return 0;
	}
}

//==============================================================================================
// 初期化
//==============================================================================================
void initializeGame(void)
{
	// テクスチャ読み込み
	UQ::getGM()->loadTexture(TEX_LOGO, "image\\logo.png");
	UQ::getGM()->loadTexture(TEX_CHARA, "image\\chara.png");
	UQ::getGM()->loadTexture(TEX_ENEMY, "image\\enemy.png");
	UQ::getGM()->loadTexture(TEX_MAP, "image\\map.png");
	UQ::getGM()->loadTexture(TEX_FONT, "image\\font.png");
	UQ::getGM()->loadTexture(TEX_GROUND, "image\\ground.png");
	UQ::getGM()->loadTexture(TEX_UNGRA, "image\\ungra.png");
	UQ::getGM()->loadTexture(TEX_FORT, "image\\fort.png");
	UQ::getGM()->loadTexture(TEX_WATER, "image\\water.png");

	// BGM読み込み
	UQ::getMM()->open(MSC_TITLE, "sound\\opening.ogg");
	UQ::getMM()->open(MSC_GROUND, "sound\\ground.ogg");
	UQ::getMM()->open(MSC_GROUND + TO_TEMPOUP, "sound\\ground_tempoup.ogg");
	UQ::getMM()->open(MSC_WATER, "sound\\water.ogg");
	UQ::getMM()->open(MSC_WATER + TO_TEMPOUP, "sound\\water_tempoup.ogg");
	UQ::getMM()->open(MSC_ATHLE, "sound\\athle.ogg");
	UQ::getMM()->open(MSC_ATHLE + TO_TEMPOUP, "sound\\athle_tempoup.ogg");
	UQ::getMM()->open(MSC_SKY, "sound\\sky.ogg");
	UQ::getMM()->open(MSC_SKY + TO_TEMPOUP, "sound\\sky_tempoup.ogg");
	UQ::getMM()->open(MSC_UNGRA, "sound\\ungra.ogg");
	UQ::getMM()->open(MSC_UNGRA + TO_TEMPOUP, "sound\\ungra_tempoup.ogg");
	UQ::getMM()->open(MSC_FORT, "sound\\fort.ogg");
	UQ::getMM()->open(MSC_FORT + TO_TEMPOUP, "sound\\fort_tempoup.ogg");
	UQ::getMM()->open(MSC_MAP, "sound\\map.ogg");
	UQ::getMM()->open(MSC_DIE, "sound\\die.ogg");
	UQ::getMM()->open(MSC_CLEAR, "sound\\clear.ogg");
	UQ::getMM()->open(MSC_BOSS, "sound\\boss.ogg");
	UQ::getMM()->open(MSC_TEMPOUP, "sound\\tempoup.ogg");
	UQ::getMM()->open(MSC_GAMEOVER, "sound\\gameover.ogg");

	// 効果音読み込み
	UQ::getSM()->load(SND_JUMP, "sound\\jump.wav");
	UQ::getSM()->load(SND_BREAK, "sound\\break.wav");
	UQ::getSM()->load(SND_SWIM, "sound\\swim.wav");
	UQ::getSM()->load(SND_DIE, "sound\\die.wav");
	UQ::getSM()->load(SND_1UP, "sound\\1up.wav");
	UQ::getSM()->load(SND_YELLOW, "sound\\yellow.wav");
	UQ::getSM()->load(SND_GREEN, "sound\\green.wav");
	UQ::getSM()->load(SND_BLUE, "sound\\blue.wav");
	UQ::getSM()->load(SND_KILL, "sound\\kill.wav");
	UQ::getSM()->load(SND_EXPLODE, "sound\\explode.wav");
	UQ::getSM()->load(SND_PAUSE, "sound\\pause.wav");
	UQ::getSM()->load(SND_PI, "sound\\pi.wav");
	UQ::getSM()->load(SND_PICOON, "sound\\picoon.wav");

	// 仮想コントローラ割り当て
	UQ::getIM()->assign(UQ::InputMgr::VC1_UP,    "Up",    UQ::InputMgr::K_UP,    UQ::InputMgr::J1_UP);
	UQ::getIM()->assign(UQ::InputMgr::VC1_DOWN,  "Down",  UQ::InputMgr::K_DOWN,  UQ::InputMgr::J1_DOWN);
	UQ::getIM()->assign(UQ::InputMgr::VC1_LEFT,  "Left",  UQ::InputMgr::K_LEFT,  UQ::InputMgr::J1_LEFT);
	UQ::getIM()->assign(UQ::InputMgr::VC1_RIGHT, "Right", UQ::InputMgr::K_RIGHT, UQ::InputMgr::J1_RIGHT);
	UQ::getIM()->assign(UQ::InputMgr::VC1_BTN1, "Jump", UQ::InputMgr::K_Z, UQ::InputMgr::J1_BTN1);
	UQ::getIM()->assign(UQ::InputMgr::VC1_BTN2, "Pick-up/Throw", UQ::InputMgr::K_X, UQ::InputMgr::J1_BTN2);
	UQ::getIM()->assign(UQ::InputMgr::VC1_BTN3, "Pause", UQ::InputMgr::K_ENTER, UQ::InputMgr::J1_BTN3);

	// スタート
	UQ::getTM()->insert(new Title());
}

