#pragma once

#include "fmfmap.h"

class Map : public UQ::Task{
public:
	enum Kind{
		EMPT, BLCK, FLOR, THRU, LADR,
		PNFL, PNTH, GRBL, GRTH,
		DOOR, DST1, DST2, DST3, DST4, DST5, DGOL, DPRV, DNXT,
		WATR, ICEB,
		KIND_MAX
	};

	struct Status{
		bool On; // 乗れる
		bool Thru; // スルーする
	};

	enum Data{
		CHARA_L = 0, CHARA_R,
		BGM_GROUND = 2, BG_GROUND = 9,
		A1 = 24, A2, B1, B2,
		LIFT_LR = 32, LIFT_UD,
		HIDE1 = 35, HIDE2, HIDE3, HIDE4, HIDE5,
		NOSCROLL_UP = 44, NOSCROLL_DOWN, NOSCROLL_LEFT, NOSCROLL_RIGHT,
		CRYSTAL = 48, ENEMY = 64, BOSS = 176
	};

public:
	Map(int door = EMPT);
	UQ::Vector2DTemplate<int> enterDoor(int door);
	void render(void);
	
	int getWidth(){return fmfMap_.GetMapWidth();}
	int getHeight(){return fmfMap_.GetMapHeight();}
	int getLayerCount(void){return fmfMap_.GetLayerCount();}
	int getValue(int layer, int x, int y){return fmfMap_.GetValue(layer, x, y);}
	int getValue(int layer, float x, float y){return fmfMap_.GetValue(layer, (int)x / 32, (int)y / 32);}
	void setValue(int layer, int x, int y, int value){fmfMap_.SetValue(layer, x, y, value);}
	Kind getKind(int x, int y);
	Kind getKind(float x, float y){return getKind((int)floor(x / 32), (int)floor(y / 32));}
	Status getStatus(int x, int y){return status_[getKind(x, y)];}
	Status getStatus(float x, float y){return status_[getKind(x, y)];}
	Status getStatus(int value){return status_[kind_[value]];}
	bool isWater(int x, int y);
	bool isWater(float x, float y){return isWater((int)floor(x / 32), (int)floor(y / 32));}
	
	static void goSelectStage(void){
		int prevStage = gameInfo.stage_;
		gameInfo.stage_ = 0;
		UQ::getTM()->insert(new Map((prevStage == 0) ? EMPT : prevStage + DST1 - 1));
	}
public:
	static const Kind kind_[256];
	static const Status status_[KIND_MAX];
	int crystalCount_;
	int bg_;
	float left_, top_;

private:
	CFmfMap fmfMap_;
};

class MapFront : public UQ::Task{
public:
	MapFront(Map* pMap) : UQ::Task(NOT_INSERT_, RENDER_MAPFRONT), pMap_(pMap){}
	void render(void);
private:
	Map* pMap_;
};
