#include "PCH.h"
#include "scene.h"
#include "chara.h"
#include "map.h"
#include <fstream>

namespace{
void drawText(const char* str, int x, int y, int power = 1, bool isDark = false){
	UQ::getGM()->setTexture(TEX_FONT);
	UQ::Polygon rect;
	rect.setColor(isDark ? 0xFF666666 : 0xFFFFFFFF);
	for(unsigned int i = 0; i < strlen(str); i++){
		rect.setRectangle(x + i * 14 * power, y, 16 * power, 16 * power);
		rect.setRectangleTexture(((str[i] - ' ') % 10) * 16, ((str[i] - ' ') / 10) * 16, 16, 16);
		UQ::getGM()->drawPolygon(rect);
	}
}

const char* getRank(int score)
{
	static const char *RANK[6] = {"S", "A", "B", "C", "D", "E"};
	return RANK[min(10 - score / 10, 5)];
}
}

//==============================================================================================
// メニュー
//==============================================================================================
Menu::Menu(int itemCount, Item item[], int top, bool isBackDark) :
	UQ::Task(UPDATE_SCENE, RENDER_SCENE),
	isEnd_(false), isSelecting_(true), cursorY_(0), top_(top), isBackDark_(isBackDark), itemCount_(itemCount)
{
	for(int i = 0; i < itemCount; i++){
		item_[i] = item[i];
	}
	if(top_ < 0) top_ = (SCREEN_HEIGHT - itemCount_ * 16) / 2;
}
void Menu::update(void)
{
	if(isSelecting_){
		if(UQ::getIM()->isPressed(UQ::InputMgr::VC1_DOWN) && cursorY_ < itemCount_ - 1){
			UQ::getSM()->play(SND_PI);
			if(item_[cursorY_ + 1].isValid) cursorY_++;
		}
		if(UQ::getIM()->isPressed(UQ::InputMgr::VC1_UP) && cursorY_ > 0){
			UQ::getSM()->play(SND_PI);
			if(item_[cursorY_ - 1].isValid) cursorY_--;
		}
		if(UQ::getIM()->isPressed(UQ::InputMgr::VC1_BTN1)){
			UQ::getSM()->play(SND_PICOON);
			isSelecting_ = false;
			if(item_[cursorY_].fades) UQ::getTM()->insert(new Fade(true));
		}
	}else{
		if(!item_[cursorY_].fades || UQ::getTM()->get<Fade>()->isEnd()) isEnd_ = true;
	}
}
void Menu::render(void)
{
	int width = strlen(item_[0].string) + 2;

	if(isBackDark_){
		UQ::getGM()->unsetTexture();
		UQ::Polygon back;
		back.setColor(0xAA000000);
		back.setRectangle((SCREEN_WIDTH - width * 14) / 2 - 16, top_ - 4, width * 14 + 32, itemCount_ * 16 + 4);
		UQ::getGM()->drawPolygon(back);
	}

	UQ::getGM()->setTexture(TEX_FONT);
	for(int i = 0; i < itemCount_; i++){
		drawText(item_[i].string, (SCREEN_WIDTH - width * 14) / 2 + 28, top_ + i * 16, 1, !(item_[i].isValid));
	}
	if(isSelecting_ || (!isSelecting_ && UQ::getSystem()->getFrameCount() / 2 % 2 == 0)){
		UQ::Polygon cursor;
		cursor.setRectangle((SCREEN_WIDTH - width * 14) / 2, top_ + cursorY_ * 16, 16, 16);
		cursor.setRectangleTexture(160, ((!isSelecting_) ? 3 : UQ::getSystem()->getFrameCount() / 6 % 6) * 16, 16, 16);
		UQ::getGM()->drawPolygon(cursor);
	}
}

//==============================================================================================
// タイトル
//==============================================================================================
Title::Title() : UQ::Task(UPDATE_SCENE, RENDER_SCENE)
{
	Menu::Item item[] = {
		{"START", true, true},
		{"QUIT" , true, true}
	};
	UQ::getTM()->insert(new Menu(ARRAY_SIZE(item), item, 360, false), this);

	gameInfo.setWorldStage(9, 0);
	UQ::getTM()->insert(new Map(), this);
	UQ::getMM()->play(MSC_TITLE, true);
}
void Title::update(void)
{
	Menu* pMenu = UQ::getTM()->get<Menu>();
	if(pMenu->isEnd()){
		erase();
		switch(pMenu->getCursorY()){
		case 0:{ // START
			std::ifstream ifs("save.dat");
			if(ifs){
				ifs.read((char*)&gameInfo, sizeof(GameInfo));
			}else{
				gameInfo.setWorldStage(1, 0);
			}
			Chara::rest_ = 3; Chara::yellow_ = 0;
			
			Map::goSelectStage();
			break;}

		case 1: // QUIT
			UQ::getSystem()->quit();
			break;
		}
	}
}
void Title::render(void)
{
	UQ::getGM()->setTexture(TEX_LOGO);
	UQ::Polygon logo;
	logo.setRectangle((SCREEN_WIDTH - 500) / 2, 20, 500, 215);
	logo.setRectangleTexture(0, 0, 500, 215);
	UQ::getGM()->drawPolygon(logo);
	drawText("UNKNOWN-QUANTITY 2010", SCREEN_WIDTH - 14 * 21 - 2, SCREEN_HEIGHT - 16);
}

//==============================================================================================
// タイマー
//==============================================================================================
void Timer::update(void)
{
	if(UQ::getTM()->get<Fade>()) return;
	if(count_ >= COUNT_MAX_) return;
	count_++;
	if(count_ == COUNT_MAX_ * 2 / 3){
		UQ::getMM()->play(MSC_TEMPOUP, false);
	}
	if(count_ == COUNT_MAX_ * 2 / 3 + 3 * 60){
		UQ::getMM()->play(bgm_ + TO_TEMPOUP, true);
	}
}
void Timer::setBgm(int bgm)
{
	bgm_ = bgm;
	if(count_ >= COUNT_MAX_ * 2 / 3 && count_ < COUNT_MAX_ * 2 / 3 + 3 * 60){
		UQ::getMM()->play(MSC_TEMPOUP, false);
	}else if(count_ >= COUNT_MAX_ * 2 / 3 + 3 * 60){
		UQ::getMM()->play(bgm_ + TO_TEMPOUP, true);
	}else{
		UQ::getMM()->play(bgm_, true);
	}
}

//==============================================================================================
// 情報
//==============================================================================================
void Info::render(void)
{
	char str[80];
	static const char *HP[4] = {"   ", "@  ", "@@ ", "@@@"};

	Chara* pChara = UQ::getTM()->get<Chara>();

	UQ::getGM()->setTexture(TEX_FONT);
	if(gameInfo.isStageSelect()){
		sprintf(str, "WORLD %d   %s & %02d  $ %02d", gameInfo.world_, HP[pChara->hp_], pChara->rest_, pChara->yellow_);
	}else{
		sprintf(str, "STAGE %d-%d %s & %02d  $ %02d  # %d", gameInfo.world_, gameInfo.stage_, HP[pChara->hp_], pChara->rest_, pChara->yellow_, pChara->green_);
	}
	drawText(str, 2, 2);
	if(!gameInfo.isStageSelect()){
		sprintf(str, "' %3d", UQ::getTM()->get<Timer>()->getRestTime());
		drawText(str, SCREEN_WIDTH - 14 * 5 - 2, 2);
	}
}

//==============================================================================================
// ステージスタート
//==============================================================================================
void StageStart::render(void)
{
	count_++;
	if(count_ > 60){
		erase();
		UQ::getTM()->insert(new Map());
	}
	char str[20];
	UQ::getGM()->setTexture(TEX_FONT);
	sprintf(str, "STAGE %d-%d START!", gameInfo.world_, gameInfo.stage_);
	drawText(str, (SCREEN_WIDTH - strlen(str) * 14) / 2, 140);
}

//==============================================================================================
// ステージクリア
//==============================================================================================
StageClear::StageClear(int mapCrystal, int charaCrystal) :
	UQ::Task(NOT_INSERT_, RENDER_SCENE),
	count_(0),
	mapCrystal_(mapCrystal), charaCrystal_(charaCrystal),
	score_(charaCrystal_ * 100 / mapCrystal_)
{
	UQ::getMM()->play(MSC_CLEAR, false);
	Chara* pChara = new Chara(-4, 12, Object::RIGHT);
	pChara->setStatus(Chara::CLEAR);
	UQ::getTM()->insert(pChara, this);
}
void StageClear::render(void)
{
	static const int COUNT_LABEL = 90;
	static const int COUNT_FRAC = COUNT_LABEL + 45;
	static const int COUNT_PAR = COUNT_FRAC + 45;
	static const int COUNT_RANK1 = COUNT_PAR + 60;
	static const int COUNT_RANK2 = COUNT_RANK1 + 60;
	static const int COUNT_NEW = COUNT_RANK2 + 30;
	static const int COUNT_END = 550;

	count_++;

	UQ::getGM()->setTexture(TEX_FONT);

	char str[20];
	sprintf(str, "STAGE %d-%d CLEAR!", gameInfo.world_, gameInfo.stage_);
	drawText(str, (SCREEN_WIDTH - strlen(str) * 14) / 2, 120);
	
	if(count_ >= COUNT_LABEL){
		drawText("GOT CRYSTAL", (SCREEN_WIDTH - 11 * 14) / 2, 184);
	}
	if(count_ >= COUNT_FRAC){
		sprintf(str, "%2d / %2d       ", charaCrystal_, mapCrystal_);
		drawText(str, (SCREEN_WIDTH - strlen(str) * 14) / 2, 216);
	}
	if(count_ >= COUNT_PAR){
		sprintf(str, "         %3d %%", score_);
		drawText(str, (SCREEN_WIDTH - strlen(str) * 14) / 2, 216);
	}
	if(count_ >= COUNT_RANK1){
		drawText("RANK", (SCREEN_WIDTH - 7 * 14) / 2, 280);
	}
	if(count_ >= COUNT_RANK2){
		drawText(getRank(score_), (SCREEN_WIDTH - 7 * 14) / 2 + 5 * 14, 264 + 3, 2);
	}
	if(count_ >= COUNT_NEW){
		if(score_ > gameInfo.getScore()){
			drawText("NEW RECORD!", (SCREEN_WIDTH - 11 * 14) / 2, 328);
		}
	}

	if(count_ == COUNT_END){
		if(score_ > gameInfo.getScore()) gameInfo.setScore(score_);
		// セーブ
		std::ofstream ofs("save.dat", std::ios_base::out | std::ios_base::trunc | std::ios_base::binary);
		ofs.write((char*)&gameInfo, sizeof(GameInfo));

		Map::goSelectStage();
		erase();
	}
}

//==============================================================================================
// ゲームオーバー
//==============================================================================================
GameOver::GameOver() : UQ::Task(NOT_INSERT_, RENDER_SCENE), count_(0)
{
	UQ::getMM()->play(MSC_GAMEOVER, false);
}
void GameOver::render(void)
{
	count_++;
	drawText("GAME OVER", (SCREEN_WIDTH - 14 * 9) / 2, max(SCREEN_HEIGHT - count_, 140));

	if(count_ == 7 * 60){
		Menu::Item item[] = {{"TITLE", true, true}};
		UQ::getTM()->insert(new Menu(ARRAY_SIZE(item), item, 300, false), this);
	
	}else if(count_ >= 7 * 60){
		Menu* pMenu = UQ::getTM()->get<Menu>();
		if(pMenu->isEnd()){
			switch(pMenu->getCursorY()){
			case 0:
				erase();
				UQ::getTM()->insert(new Title());
				break;
			}
		}
	}
}

//==============================================================================================
// ポーズ
//==============================================================================================
Pause::Pause() : UQ::Task(UPDATE_SCENE, RENDER_SCENE)
{
	if(gameInfo.isStageSelect()){
		Menu::Item item[] = {
			{"PLAY ", true, false},
			{"TITLE", true, true}
		};
		UQ::getTM()->insert(new Menu(ARRAY_SIZE(item), item, (SCREEN_HEIGHT - 16 * 8) / 2, true), this);
	}else{
		Menu::Item item[] = {
			{"PLAY", true, false},
			{"EXIT", (gameInfo.getScore() >= 0), true}
		};
		UQ::getTM()->insert(new Menu(ARRAY_SIZE(item), item, -1, true), this);
	}
	
	UQ::getSM()->play(SND_PAUSE);
	UQ::getMM()->setVolume(0.3f);
	UQ::getTM()->get<Map>()->pauseUpdate();
}
void Pause::update(void)
{
	Menu* pMenu = UQ::getTM()->get<Menu>();
	if(pMenu->isEnd()){
		erase();
		UQ::getMM()->setVolume(1.0f);
		if(gameInfo.isStageSelect()){
			switch(pMenu->getCursorY()){
			case 0:
				UQ::getTM()->get<Map>()->resumeUpdate();
				break;
			case 1:
				UQ::getTM()->get<Map>()->erase();
				UQ::getTM()->insert(new Title());
				break;
			}

		}else{
			switch(pMenu->getCursorY()){
			case 0:
				UQ::getTM()->get<Map>()->resumeUpdate();
				break;
			case 1:
				UQ::getTM()->get<Map>()->erase();
				Map::goSelectStage();
				break;
			}
		}
	}
}
void Pause::render(void)
{
	if(gameInfo.isStageSelect()){
		UQ::getGM()->unsetTexture();
		UQ::Polygon rect;
		rect.setColor(0xAA000000);
		rect.setRectangle((SCREEN_WIDTH - 13 * 14) / 2 - 16, (SCREEN_HEIGHT - 8 * 16) / 2 + 3 * 16 - 4, 13 * 14 + 32, 5 * 16 + 4);
		UQ::getGM()->drawPolygon(rect);
		for(int i = 1; i <= 5; i++){
			int& score = gameInfo.score_[gameInfo.world_ - 1][i - 1];
			char str[60];
			if(score < 0){
				sprintf(str, "%d-%d    - %%  -", gameInfo.world_, i);
			}else{
				sprintf(str, "%d-%d  %3d %%  %s", gameInfo.world_, i, score, getRank(score));
			}
			drawText(str, (SCREEN_WIDTH - 14 * 13) / 2, (SCREEN_HEIGHT - 16 * 8) / 2 + 32 + i * 16);
		}
	}
}

//==============================================================================================
// フェード
//==============================================================================================
Fade::Fade(bool isFadeOut) : UQ::Task(NOT_INSERT_, RENDER_FADE), count_(0), isFadeOut_(isFadeOut)
{
	if(isFadeOut) UQ::getMM()->stop();
}
void Fade::render(void)
{
	count_++;
	if(count_ == COUNT_MAX_) erase();

	UQ::getGM()->unsetTexture();
	UQ::Polygon rect;
	rect.setColor(UQ::Color(((!isFadeOut_) ? (COUNT_MAX_ - count_) : count_) * 0xFF / COUNT_MAX_, 0, 0, 0));
	rect.setRectangle(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
	UQ::getGM()->drawPolygon(rect);
}
